/***********************************************************************/
/* w_noise.c  - C5x  White noise generator                             */
/***********************************************************************/
#include <stdlib.h>

#include "dskaic5x.h"
#include "c5xregs.h"
#include <ioports.h>

#define    CLEAR_INT_MASK_BIT          asm("\t clrc\t INTM")
/* Function prototypes for forwards */
void initDSP(void);

AIC_PRIMARY aicPrimary;


char polynominal =0x8805 /*wielomian 16 */ /* 0x4001 wielomian 15 */ ,
    shift_reg = 0x8801;  /* rejestr przesuwny */ 


/***********************************************************************/
/* MAIN()                                                              */
/***********************************************************************/
void main(void)
{
     int i;
     unsigned int L;

     /* AIC paramaters for overriding the default value (optional) */
     AIC_CONFIGURATION aicParams;


     /* Initialize DSP environment */
     initDSP();

     /* Initialize aic primary word data structure */
     aicPrimary._bitval.command = 0;
     aicPrimary._bitval.data    = 0;

     /* Get default aic paramaters */
     /* Set AIC in loopback mode */
     getDefaultAicConfig(&aicParams);
     initAic(&aicParams);     /* Use default AIC configuration */



     for(;;) {
           outport(0,++L^0xFFFF);
     }       /* Loop forever (processing done in ISR) */
}

/***********************************************************************/
/* INITINTS() - Initialize interrupt structure                         */
/*              - clear all pending interrupts                         */
/*              - mask all interrupts                                  */
/*              - clear all pending interrupts                         */
/*              - globally disable all interrupts                      */
/***********************************************************************/
void initDSP(void)
{

/*
* Initialize wait state generators
*

     CWSR = BIG;
     PDWSR = 0x0;
     IOWSR =0;

* Initialize the Interrupt Environment
*  Clear int mask bit in the status register, globally disabling all int's
*  Clear imr (interrupt mask) register, disabling all int's
*  Clear ifr (interrupt flag) register, clearing all pending int's
*/
     CLEAR_INT_MASK_BIT;
     IMR = 0x0;
     IFR = 0xffff;
#ifdef sim
  PMST = 0x808;
#endif
/*     PMST |= RAM | OVLY | MP_MC; */
}

/* Create descriptive names for the serial port ISR's */
#define serialPortReceiveISR           c_int5

/***********************************************************************/
/* serialPortReceiveISR - Serial port interrupt handler                */
/*                                                                     */
/***********************************************************************/
void serialPortReceiveISR(void)
{
    char cnt, out = 0,
        temp = shift_reg & polynominal;

    *DRR;
    for(cnt = 0; cnt < 16; cnt++)  /* obliczanie wartosci wsuwanego bitu 
				      na podstawie wielomianu i aktualnej 
				      wartosci rejestru                 */
    {
        out ^= temp & 0x0001;
        temp >>= 1;
    }
    shift_reg <<= 1;
    shift_reg |= out;

    *DXR = shift_reg;  /* aby dzialalo na plytce przesunac w lewo  */
}

/***********************************************************************/
/* dummy()   - dummy interrupt handler, used for debugging             */
/***********************************************************************/
void dummy(void)
{
     for(;;) {};
}
