/***********************************************************************/
/* FIR.c  - C5x   FIR filter program (21 taps)                         */
/***********************************************************************/
#include <stdlib.h>

#include "dskaic5x.h"
#include "c5xregs.h"
#include <ioports.h>
#include <math.h>
#define    CLEAR_INT_MASK_BIT          asm("\t clrc\t INTM")
#define N 11

static long hist[N]={0,0,0,0,0,0,0,0,0,0,0};

/* Function prototypes for forwards */
void initDSP(void);

AIC_PRIMARY aicPrimary;

/***********************************************************************/
/* MAIN()                                                              */
/***********************************************************************/
void main(void)
{
     unsigned int L;
     int i;
     /* AIC paramaters for overriding the default value (optional) */
     AIC_CONFIGURATION aicParams;


     /* Initialize DSP environment */
     initDSP();

     /* Initialize aic primary word data structure */
     aicPrimary._bitval.command = 0;
     aicPrimary._bitval.data    = 0;

     /* Get default aic paramaters */
     /* Set AIC in loopback mode */
     getDefaultAicConfig(&aicParams);

     aicParams.command2.tb = 15;   /* sampling 19,2kHz */
     aicParams.command2.rb = 15;

     initAic(&aicParams);     /* Use default AIC configuration */

     for(;;)   outport(0,++L^0xFFFF);  /* Loop forever (processing done in ISR) */
}
/***********************************************************************/
/* INITINTS() - Initialize interrupt structure                         */
/*              - clear all pending interrupts                         */
/*              - mask all interrupts                                  */
/*              - clear all pending interrupts                         */
/*              - globally disable all interrupts                      */
/***********************************************************************/
void initDSP(void)
{

/*
* Initialize wait state generators
*

     CWSR = BIG;
     PDWSR = 0x0;
     IOWSR =0;

* Initialize the Interrupt Environment
*  Clear int mask bit in the status register; globally disabling all int's
*  Clear imr (interrupt mask) register; disabling all int's
*  Clear ifr (interrupt flag) register; clearing all pending int's
*/
     CLEAR_INT_MASK_BIT;
     IMR = 0x0;
     IFR = 0xffff;
#ifdef sim
     PMST = 0x800;
#endif
}

/* Create descriptive names for the serial port ISR's */
#define serialPortReceiveISR           c_int5

/***********************************************************************/
/* serialPortReceiveISR - Serial port receive interrupt handler        */
/*                        calculates output signal                     */
/***********************************************************************/

void serialPortReceiveISR(void)
{
  static int coef[N]={6553,17645,12425,9152,6206,3230,
                      439,-1753,-3047,-3404,0};
  static long histnew[N];

    int i;
    long output;

/* form output accumulation */

    output = coef[0] * ((int)*DRR>>1);
    histnew[0] = output;

    for(i = 1 ; i < N ; i++)
       {
         histnew[i] = (long) coef[i] * (output>>15) + (hist[i-1]<<15);
         output += (long) coef[i] * (long) hist[i-1];
         hist[i-1] = (histnew[i-1]>>15);
       }

    hist[i-1] = (histnew[i-1]>>15);

    *DXR = (output>>15) & 0xfffc;
}

/***********************************************************************/
/* dummy()   - dummy interrupt handler; used for debugging             */
/***********************************************************************/
void dummy(void)
{
     for(;;) {};
}
