function   x = tdchirp( T, W, p, Td )
%CHIRP    generate a sampled chirp signal with delay Td
%-----       exp(j(W/T)pi*(t-ss)^2)   -T/2+ss <= t < +T/2+ss
%
%   Usage:   X = tdchirp( T, W, <Td>, <P> )
%
%      X :  N=pTW samples of a "chirp" signal
%      T :  time duration from -T/2 to +T/2
%      W :  swept bandwidth from -W/2 to +W/2
%      Td:  delay of a pulse unit of 1/(pW)
%
%   optional (default is P = 1 as well as Td)
%      P :  samples at P times the Nyquist rate (W)
%            i.e., sampling interval is 1/(PW)

%---------------------------------------------------------------
% copyright 1994, by C.S. Burrus, J.H. McClellan, A.V. Oppenheim,
% T.W. Parks, R.W. Schafer, & H.W. Schussler.  For use with the book
% "Computer-Based Exercises for Signal Processing Using MATLAB"
% (Prentice-Hall, 1994).
%---------------------------------------------------------------

if nargin <= 3
   Td = 0;   end    
if nargin <= 2
   p=1;   end    
J = sqrt(-1);
%--------------
delta_t = 1/(p*W);
N = round( p*T*W );    %--- same as T/delta_t
if Td<0
	disp('Nie obsluguje przyspieszenia ustawiam Td=0');
	Td=0;
end
if Td==0
	nn = [0:N-1]';
	arg=(delta_t*nn - T/2 ) ;
	x = exp( J*pi*W/T * (delta_t*nn - T/2 ).^2 );
elseif Td>0
	ad=floor(Td);
	ad=ceil(Td);
	nn = [ad:N+ad-1];
	x = exp( J*pi*W/T * ((nn -Td)*delta_t - T/2 ).^2 );
	x=[zeros([1 ad]) x].';
end

