/**********************************************************************
* FILENAME
*   sinus.c
*
* DESCRIPTION
*  Sinus generator.  
*  Jecek Majewski ICT,PWr, 01.03.2001
*
* Program based on 
* Rev 2.1	03 Jun 2000	 Z.Zhang, T.J.Dillon 
**********************************************************************/
#define DSK 0  

#include <stdlib.h>
#include <stdio.h>
#include <c6x.h>
#include <math.h>
#include "c6211dsk.h"
#include "codec.h"
typedef  unsigned volatile int   uv_int;
#define KEYS    (*(uv_int *)IO_PORT)
#define LEDS     *(uv_int *)IO_PORT
void codec_init();
int temp,in,out;   
#define N 15.3


int main()
{
    double A=2*(cos(2.0*PI/N));
    double C=sin(2.0*PI/N);  
    double y[3];
 	
  /* dsp and periphiral initialization */    
  CSR=0x100;	        	       /* disable all interrupts            */
  IER=1;                           /* disable all interrupts except NMI */
  ICR=0xffff;                      /* clear all pending interrupts      */

  *(uv_int *)EMIF_GCR = 0x3300; 	     /* EMIF global control               */
  *(uv_int *)EMIF_CE1 = CE1_32;	 /* EMIF CE1 control, 8bit async      */
#if DSK 
  mcbsp0_init();
  codec_init();
#endif    
  y[1]=C;
  y[0]=y[2]=0;
 

 while(1)  /* play back forever */{
    y[0] = A * y[1]  - y[2];
    y[2] = y[1];
    y[1] = y[0];   
    out =((long) (y[0]*32768)) & 0xfffe ;  
    #if DSK 
     mcbsp0_write(out);
    #endif
  };
  return(0);

}

void mcbsp0_init()
{
     /* set up McBSP0                                                                   */
    *(uv_int *)McBSP0_SPCR = 0;			        /*  reset serial port                         */
    *(uv_int *)McBSP0_PCR = 0;              /*  set pin control reg.;                     */
    *(uv_int *)McBSP0_RCR = 0x10040;        /*  set rx control reg. one 16 bit data/frame */
    *(uv_int *)McBSP0_XCR = 0x10040;        /*  set tx control reg. one 16 bit data/frame */
    *(uv_int *)McBSP0_DXR = 0;
    *(uv_int *)McBSP0_SPCR = 0x12001;       /*  setup SP control reg.;                    */
}

void mcbsp0_write(int out_data)
{
  int temp;
  temp = *(uv_int *)McBSP0_SPCR & 0x20000;
  while ( temp == 0)
  {
    temp = *(uv_int *)McBSP0_SPCR & 0x20000;
  }
  *(uv_int *)McBSP0_DXR = out_data;
}

int mcbsp0_read()
{
  int temp;
  temp = *(uv_int *)McBSP0_SPCR & 0x2;
  while ( temp == 0)
  {
    temp = *(uv_int *)McBSP0_SPCR & 0x2;
  }
  temp = *(uv_int *)McBSP0_DRR;
  return temp;
}
 

void codec_init()
{
  int temp;
  
  // set up control register 3 for S/W reset
  mcbsp0_read();
  mcbsp0_write(0); 
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0386);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
   
  // set up control register 3 for mic input
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0306);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();

  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x2330);
  temp = mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  if((temp & 0xff) != 0x06) 
  {
#if PRINT
    printf ("Error in setting up register 3.\n");
    exit(0);
#endif
  }

//set up control register 4
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0400);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();

  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x2430);
  temp = mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  if((temp & 0xff) != 0x00) 
  {
#if PRINT
    printf ("Error in setting up register 4.\n");
    exit(0);
#endif
  }
   
// set up control register 5
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0502);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();

  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x2530);
  temp = mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  if((temp & 0xfe) != 0x2) 
  {
#if PRINT
    printf ("Error in setting up register 5.\n");
    exit(0);
#endif
  } 
       
}


