/**********************************************************************
* FILENAME
*   dualtone.c
*
* DESCRIPTION
*  Dual tone generator.  Tones depend on switches on board.
*  Jecek Majewski ICT,PWr, 01.03.2001
*
* Program based on 
* Rev 2.1	03 Jun 2000	 Z.Zhang, T.J.Dillon 
**********************************************************************/
#define DSK 1 

#include <stdlib.h>
#include <stdio.h>
#include <c6x.h>
#include <math.h>
#include "c6211dsk.h"
#include "codec.h"
#define Keys    (((*(unsigned volatile int *)IO_PORT)>>24) & 0x07)
void codec_init();
int in,out;   
 
int main()
{
  int temp, previous, next, row, column;
  const  double f[8]= {12.09, 13.36, 14.77,16.33, 6.97, 7.70, 8.52, 9.51};
  double A[8],C[8]; 
  double y1[3],y2[3];
 	
  /* dsp and periphiral initialization */    
  CSR=0x100;	        	       /* disable all interrupts            */
  IER=1;                           /* disable all interrupts except NMI */
  ICR=0xffff;                      /* clear all pending interrupts      */

  *(unsigned volatile int *)EMIF_GCR = 0x3300; 	     /* EMIF global control               */
  *(unsigned volatile int *)EMIF_CE1 = CE1_32;	 /* EMIF CE1 control, 8bit async      */
#if DSK 
  mcbsp0_init();
  codec_init();
#endif    
 
  for( temp=0; temp<8; temp++){
      A[temp]=2*cos(2.0*PI/f[temp]/0.8);     // fsample=8kHz
      C[temp]=sin(2.0*PI/f[temp]/0.8);       
  }
  		
	y1[1]=y1[0]=y1[2]=0;
	y2[1]=y2[0]=y2[2]=0;
	
    previous=  Keys;
 
    while(1)  /* play back forever */{
     *(unsigned volatile int *)IO_PORT= (*(unsigned volatile int *)IO_PORT) &  0x07000000;
   
     next= Keys;  
     if (previous != next ){ 
    	 previous =next;
   		 row=next & 0x3;  
   		 column=(next >>2) + 4;
	     y1[1]=C[row];
	     y1[0]=y1[2]=0;
	     y2[1]=C[column];
 	     y2[0]=y2[2]=0;  
 	     if(next==7) y1[1]=y2[1]=0;   // silence if keys off
   	} 
    /* first sinus */      
    y1[0] = A[row]* y1[1]  - y1[2];
    y1[2] = y1[1];
    y1[1] = y1[0];
    /* second sinus */
    y2[0] = A[column] * y2[1]  - y2[2];
    y2[2] = y2[1];
    y2[1] = y2[0];
    
     /* output */    
     out =((int) ((y1[0]+y2[0])*65536/4)) & 0xfffe ;  
    #if DSK 
     mcbsp0_write(out);
    #endif
  };
//  return(0);

}

void mcbsp0_init()
{
     /* set up McBSP0                                                                   */
    *(unsigned volatile int *)McBSP0_SPCR = 0;			        /*  reset serial port                         */
    *(unsigned volatile int *)McBSP0_PCR = 0;              /*  set pin control reg.;                     */
    *(unsigned volatile int *)McBSP0_RCR = 0x10040;        /*  set rx control reg. one 16 bit data/frame */
    *(unsigned volatile int *)McBSP0_XCR = 0x10040;        /*  set tx control reg. one 16 bit data/frame */
    *(unsigned volatile int *)McBSP0_DXR = 0;
    *(unsigned volatile int *)McBSP0_SPCR = 0x12001;       /*  setup SP control reg.;                    */
}

void mcbsp0_write(int out_data)
{
  int temp;
  temp = *(unsigned volatile int *)McBSP0_SPCR & 0x20000;
  while ( temp == 0)
  {
    temp = *(unsigned volatile int *)McBSP0_SPCR & 0x20000;
  }
  *(unsigned volatile int *)McBSP0_DXR = out_data;
}

int mcbsp0_read()
{
  int temp;
  temp = *(unsigned volatile int *)McBSP0_SPCR & 0x2;
  while ( temp == 0)
  {
    temp = *(unsigned volatile int *)McBSP0_SPCR & 0x2;
  }
  temp = *(unsigned volatile int *)McBSP0_DRR;
  return temp;
}
 

void codec_init()
{
  int temp;
  
  // set up control register 3 for S/W reset
  mcbsp0_read();
  mcbsp0_write(0); 
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0386);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
   
  // set up control register 3 for mic input
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0306);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();

  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x2330);
  temp = mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  if((temp & 0xff) != 0x06) 
  {
#if PRINT
    printf ("Error in setting up register 3.\n");
    exit(0);
#endif
  }

//set up control register 4
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0400);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();

  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x2430);
  temp = mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  if((temp & 0xff) != 0x00) 
  {
#if PRINT
    printf ("Error in setting up register 4.\n");
    exit(0);
#endif
  }
   
// set up control register 5
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x0502);
  mcbsp0_read();
  mcbsp0_write(0);
  mcbsp0_read();

  mcbsp0_write(0);
  mcbsp0_read();
  mcbsp0_write(1);
  mcbsp0_read();
  mcbsp0_write(0x2530);
  temp = mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  mcbsp0_write(0x0);
  mcbsp0_read();
  if((temp & 0xfe) != 0x2) 
  {
#if PRINT
    printf ("Error in setting up register 5.\n");
    exit(0);
#endif
  } 
       
}


